      Program  Semiparametric_multilevel_Binomial_model
	USE      IMSL

c     10-28-2004, Junfeng Liu

c     fixed effects - bet, dimension =4 
c     random effects - e(i), i=1,2,3,4 (nc), dimension =4

      integer   p,pe,nc                !  covariate vector dimension, group (cluster) number  
	integer*4 iseed,nd,inner
	integer   irank,nr                    ! nd ~ outer loops,  nr ~ inner loops
	parameter (p=4,pe=1,nc=4,nr=10000)
	integer   m(nc),n(nc,5),y(nc,5),mo(p) ! m ~ cluster size,  mo ~ model selection
      real      x(nc,5,p),cov(p,p),rc(p,p),bet_old(p),bet_new(p),
     & 	      e(nc,pe),
     & 	      we(pe),wnb(nr,p),D(p,p),rD(p,p),run(nr*(nc*3+1)),
     &          value_old,value_new,g0(nr*nc,p),
     &          wg(nc,p),ps(nc),ps1(nc+1),ps2(nc+1),nlw,M0

      integer   in(nc),nl(nc),km,km_,wnl(nc)

	common    /dx / x,z
      common    /mny/   m,n,y

c     x       - design matrix
c     cov     - random walk proposal density for beta vector sampling
c     bet_old - old samples of beta vector
c     bet_new - new samples of beta vector
c     e       - random effects across clusters (groups)
c     km      - cardinality of current random effects 
c     km_     - temporary number of clusters for sigle element cluster 
c     g       - gamma vectors in MacEachern and Muller method
c     wnb     - random walk vector for beta vector updating
c     D       - base measure variance matrix for Nv(0,D) in Dirichelet process prior 
c     mo      - Model selection array

	M0=200            ! similarity degree to ``base measure" of Dirichlet process mixture
      iseed=12357
      
	m(1)=5
	m(2)=3
	m(3)=5
	m(4)=3

	mo(1)=1
	mo(2)=1
	mo(3)=1
	mo(4)=1

	do i=1,p          ! base measure for Dorichlet process: Nv(0,D)
  	   do j=1,p
	      D(i,j)=0.0
	      if(i.eq.j)D(i,j)=1.0D-4
	   end do
	end do

      open(3,file='beta.txt',status='unknown')
      open(4,file='epsilon.txt',status='unknown')

c      open(8,file='test.txt',status='unknown')
      
 	do i=1,p
  	   do j=1,p
	      cov(i,j)=0.0
	      if(i.eq.j)cov(i,j)=1.0D-2
	   end do
	end do
      
      open(7,file='input.txt',status='old')
      do k=1,m(1)
	   read(7,*)(x(1,k,j),j=1,p),n(1,k),y(1,k) 
	end do
      do k=1,m(2)
	   read(7,*)(x(2,k,j),j=1,p),n(2,k),y(2,k) 
	end do
      do k=1,m(3)
	   read(7,*)(x(3,k,j),j=1,p),n(3,k),y(3,k) 
	end do
      do k=1,m(4)
	   read(7,*)(x(4,k,j),j=1,p),n(4,k),y(4,k) 
	end do
      close(7)


c     initialize the whole Gibbs sampler

	km=1                       ! initial unique random effects among the 4 ncs
      do k=1,km
	   do j=1,pe
	      e(k,j)=0.0           ! initialize the unique random effect
	   end do
	end do

      do j=1,p
	   bet_old(j)=0.0
	   if(mo(j).eq.0)bet_old(j)=0.0
      end do

      do i=1,nc   ! membership set index initialization
	   in(i)=1  ! which random effect this group "i" belongs to?
	   nl(i)=0  ! cardinality of each random effect (how many groups?)
	end do

      nl(1)=4     ! cardinality of the first membership set (including all groups)

c     produce multivariate normal random vectors for beta vector sampling

	nd=0

1	call chfac(p,cov,p,0.00001,irank,rc,p)
      iseed=iseed+2
	call rnset(iseed)
	call rnmvn(nr,p,rc,p,wnb,nr)    !  for beta vector proposal
      
	do j=1,p
	   if(mo(j).eq.0)then
	      do k=1,nr
		     wnb(k,j)=0.0
            end do
         end if
      end do

c     produce base measure random effects for backup  ``phi(E)" set
 
      call chfac(p,D,p,0.00001,irank,rD,p)
      iseed=iseed+2
	call rnset(iseed)
	call rnmvn(nr*nc,p,rD,p,g0,nr*nc)

	do j=1,p
	   if(mo(j).eq.0)then
	      do k=1,nr*nc
		     g0(k,j)=0.0
            end do
         end if
      end do

	iseed=iseed+2
	call rnset(iseed)  
      call rnun(nr*(nc*3+1),run)        ! for beta vector acceptance 

      do inner=1,nr             ! the inner loop starts
      
  	   nd=nd+1

         if(km.lt.nc)then
            do k=km+1,nc
	         do j=1,pe
			    e(k,j)=g0(nc*(inner-1)+k,j)
	            if(mo(j).eq.0)e(k,j)=0.0
               end do
            end do
	   end if
	   
c     calculate old beta vector log-likelihood 

         value_old=0.0
	   do i=1,nc
	      do j=1,pe
	         we(j)=e(in(i),j)
	      end do
	      value_old=value_old+fl(i,bet_old,we,p,pe) 
  	   end do

c	w1=w1+nbet(bet0,p)    ! noninformative prior

c     calculate new beta vector log-likelihood 

  	   do j=1,p
	      bet_new(j)=bet_old(j)+wnb(inner,j)
         end do
	
         value_new=0.0
	   do i=1,nc
	      do j=1,pe
	         we(j)=e(in(i),j)
	      end do
            value_new=value_new+fl(i,bet_new,we,p,pe) 
	   end do

c	w2=w2+nbet(bet1,p)    ! noninformative prior

c     accept or reject new beta vector? 

         if(run((inner-1)*(nc*3+1)+1).lt.1.0d-10)then
	      run((inner-1)*(nc*3+1)+1)=run((inner-1)*(nc*3+1)+1)+1.0D-10
	   end if
	   
	   if(log(run((inner-1)*(nc*3+1)+1)).
     &	  le.min(0.0,value_new-value_old))then
	      do j=1,p
	         bet_old(j)=bet_new(j)
	      end do
         end if

c     sample individual random effects across cluster (group)s         
c     initialize backup random effects plus available ones (i=1,...,l)

         if(km.lt.nc)then   
            do k=km+1,nc                ! nc=4 (cluster number)
	         do j=1,pe
	            e(k,j)=g0((inner-1)*nc+k,j)
               end do
            end do
	   end if

         do i=1,nc                      ! sample random effects { e(1),...,e(nc=4) }

            if(nl(in(i)).gt.1)then      ! nl > 1, and "i" is in set "l"

c  	km is less than nc(group/cluster number) certainly
   
               do k=1,km
	            if(in(i).eq.k)nlw=nl(k)*1.0-1.0
                  if(in(i).ne.k)nlw=nl(k)*1.0
 		        do j=1,pe
  	               we(j)=e(k,j)       ! manipulate one of the random effects
				end do
                  ps(k)=nlw*f(i,bet_old,we,p,pe) !  sampling weights
               end do

c     calculate weight for first backup random effect { e(km+1) vector }

		     do j=1,pe
	            we(j)=e(km+1,j)
               end do                   
               ps(km+1)=(1.0*M0)/(km+1.0)*f(i,bet_old,we,p,pe)
		     
			 w1=0.0
	         do j=1,km+1
		        w1=w1+ps(j)
		     end do 

	         do j=1,km+1   !  usable random effects
			    ps(j)=ps(j)/w1
		     end do 
		     do k=1,km+1
		        ps2(k)=0.0
			    do j=1,k
			       ps2(k)=ps2(k)+ps(j)   ! each right cumulative probability 
                  end do
                  ps1(k)=ps2(k)-ps(k)      ! each left cumulative probability 
               end do

               do k=1,km
		        if(run((inner-1)*(nc*3+1)+i).ge.ps1(k).
     &			   and.run((inner-1)*(nc*3+1)+i).lt.ps2(k))then
                     nl(in(i))=nl(in(i))-1
                     in(i)=k
	               nl(k)=nl(k)+1
			    end if 
               end do

		     if(run((inner-1)*(nc*3+1)+i).ge.ps1(km+1).
     &			and.run((inner-1)*(nc*3+1)+i).lt.ps2(km+1))then
                  nl(in(i))=nl(in(i))-1
                  in(i)=km+1
	            nl(km+1)=1
	            km=km+1
			 end if 

c               write(*,*)'- '
c               write(*,'(4f8.3)')(e(l,1),l=1,km)
c               write(*,'(4i8)')(in(l),l=1,nc)
c               write(*,'(4i8)')(nl(l),l=1,km)
c               read(*,*) 

            end if                      ! { nl(in(i)).gt.1 }

            if(nl(in(i)).eq.1)then      ! nl = 1, and "i" is in set "l"

		     if(run((inner-1)*(nc*3+1)+nc+i).gt.1.0*km/(km+1.0)
     &			.and.km.le.nc)then

c     set membership exchange: set ``in(i)" and ``km" 
 
 		        do j=1,pe
  	               we(j)=e(in(i),j)       
  	               e(in(i),j)=e(km,j)     
  	               e(km,j)=we(j)         
				end do
	            
	            nl(in(i))=nl(km)                  ! exchange two sets 
				nl(km)=1
                  do j=1,nc
                     if(in(j).eq.km)in(j)=in(i)
				end do
                  in(i)=km

				km_=km-1                          ! clusters decreases for sure

                  do k=1,km_
                     nlw=nl(k)*1.0
c                     if(in(i).eq.k)nlw=nl(k)*1.0-1
 		           do j=1,pe
  	                  we(j)=e(k,j)       ! manipulate one of the random effects
				   end do
                     ps(k)=nlw*f(i,bet_old,we,p,pe) ! sampling weights
                  end do

c     calculate sampling weight for first backup random effect { e(km+1) vector }

		        do j=1,pe
	               we(j)=e(km_+1,j)
                  end do                 
                  ps(km_+1)=(1.0*M0)/(km_+1.0)*f(i,bet_old,we,p,pe)   ! km*=km-1
			    w1=0.0
	            do j=1,km_+1
		           w1=w1+ps(j)
		        end do 
	            do j=1,km_+1   !  usable random effects
			       ps(j)=ps(j)/w1
		        end do 
		        do k=1,km_+1
		           ps2(k)=0.0
			       do j=1,k
			          ps2(k)=ps2(k)+ps(j)   ! each right cumulative probability 
                     end do
                     ps1(k)=ps2(k)-ps(k)      ! each left cumulative probability 
                  end do

c     If choose set [km+1]= original km, then the preceding relabeling kept the full random-effect set unchanged. 
c     If choose set out of [1,...,km]

				do k=1,km_
		           if(run((inner-1)*(nc*3+1)+2*nc+1+i).ge.ps1(k).
     &	  		      and.run((inner-1)*(nc*3+1)+2*nc+1+i).lt.ps2(k))
     &                  then
       
					  in(i)=k
					  nl(k)=nl(k)+1
                        km=km-1

c                        write(*,*)'-- '
c                        write(*,'(4f8.3)')(e(l,1),l=1,km)
c                        write(*,'(4i8)')(in(l),l=1,nc)
c                        write(*,'(4i8)')(nl(l),l=1,km)
c                        read(*,*) 
           
   		           end if 
                  end do

		        if(run((inner-1)*(nc*3+1)+2*nc+1+i).ge.ps1(km_+1).
     &			   and.run((inner-1)*(nc*3+1)+2*nc+1+i).lt.ps2(km_+1))
     &               then

c     move random effects back into cluster ``km" formerly into cluster ``in(i)"

                     do j=1,pe
	                  we(j)=e(km_+1,j)
	                  e(km_+1,j)=e(in(i),j)
	                  e(in(i),j)=we(j)
				   end do

c                     write(*,*)'--- '
c                     write(*,'(4f8.3)')(e(l,1),l=1,km)
c                     write(*,'(4i8)')(in(l),l=1,nc)
c                     write(*,'(4i8)')(nl(l),l=1,km)
c                     read(*,*) 

 		        end if 


			 end if
            end if                !   { nl(in(i)).eq.1 }
        end do                   !   sample random effects { e(1),...,e(nc=4) }
	
        if(mod(nd,5000).eq.0)then
	     write(*,*)nd/5000
	  	 write(*,100)(bet_old(j),j=1,4)
	     write(*,100)((e(i,j),j=1,pe),i=1,4)
	  	 write(3,100)(bet_old(j),j=1,4)
	     write(4,100)((e(i,j),j=1,pe),i=1,4)
        end if
	
c        write(8,'(i5,4f8.3,4i3,2x,4i3,i4)')nd,(e(i,1),i=1,nc),
c     &           (in(k),k=1,nc),(nl(k),k=1,nc),km

	end do     !  inner loop ends for (nr=1,10000)

      if(nd.lt.1D08) goto 1	   	

      close(3)
      close(4)
	     
100   format(4f12.6)

	end

      real function   fl(i,bet,e,p,pe) 
	integer         nc,i,p,pe
	parameter       (nc=4)
	integer         m(nc),n(nc,5),y(nc,5)
	real            x(nc,5,4),bet(p),z(nc,5,4),e(pe),w1,w2,w3,w4

      common /dx/     x,z
      common /mny/    m,n,y
      
      do k=1,m(i)
	   do j=1,pe
	      z(i,k,j)=x(i,k,j)
	   end do
	end do
      
	fl=0.0
	do k=1,m(i)
         w1=0.0
         do j=1,p
	      w1=w1+x(i,k,j)*bet(j)
	   end do
         w2=0.0
         do j=1,pe
	      w2=w2+z(i,k,j)*e(j)
	   end do
         fl=fl+y(i,k)*(w1+w2)-n(i,k)*log(1.0+exp(w1+w2))
      end do
	return
      end

      real function   f(i,bet,e,p,pe) 
	integer         nc,i,p,pe
	parameter       (nc=4)
	integer         m(nc),n(nc,5),y(nc,5)
	real            x(nc,5,4),bet(p),z(nc,5,4),e(pe),w1,w2,w3,w4

      common /dx/     x,z
      common /mny/    m,n,y
      
      do k=1,m(i)
	   do j=1,pe
	      z(i,k,j)=x(i,k,j)
	   end do
	end do
      
	f=0.0
	do k=1,m(i)
         w1=0.0
         do j=1,p
	      w1=w1+x(i,k,j)*bet(j)
	   end do
         w2=0.0
         do j=1,pe
	      w2=w2+z(i,k,j)*e(j)
	   end do
         f=f+y(i,k)*(w1+w2)-n(i,k)*log(1.0+exp(w1+w2))
     &		 +facto(n(i,k),y(i,k))
      end do
	f=exp(f)

	return
      end

      real function nbet(bet,p)
	integer p
      real bet(p),mu(p),sig(p,p),siginv(p,p),w1(p)

c     mu0 - mean in beta vector prior
c     sig - Sigma in beta vector prior
      
	do i=1,p
	   mu(i)=0.0
	end do

	do i=1,p
	do j=1,p
	   sig(i,j)=0.0
	   if(i.eq.j)then 
	      sig(i,j)=0.01
         end if
	end do
	end do

	call dlinrg(p,sig,p,siginv,p)  
       
	do i=1,p
	   w1(i)=0.0
	   do j=1,p
	      w1(i)=w1(i)+(bet(j)-mu(j))*(siginv(j,i))
	   end do 
	end do  

      nbet=0.0
	do i=1,p
	   nbet=nbet+(bet(i)-mu(i))*w1(i)
	end do  
      nbet=-0.5*nbet
      return
	end

      real function facto(n,y)
	integer y,n

      if(y.eq.0.or.y.eq.n)then
	   facto=1.0
	else 
         c1=0.0
   	   do k=1,n
	      c1=c1+log(k*1.0)
	   end do
         c2=0.0
	   do k=1,y
	      c2=c2+log(k*1.0)
	   end do
         c3=0.0
	   do k=1,n-y
	      c3=c3+log(k*1.0)
	   end do
	   facto=c1-c2-c3
      end if
      return 
	end