      Program  Multilevel_Binomial_Regression_Gibbs_Sampler
	USE      IMSL

c     04-30-2007, Junfeng Liu
c     logit Link, Skew random effect modeling

c     fixed effects - bet, dimension =2 
c     random effects - e(i), i=1,2,3,4 (nc), dimension =1

      integer   p,nc,nb
	integer*4 iseed,irank,nd,nr
	parameter (p=2,nc=8,nb=4,nr=10000)   ! # of covariates, clusters, binomials, iterations
	integer   m(nc),n(nc,nb),y(nc,nb)
	real      x(nc,nb,p)
	real      u_old(nc),u_new(nc),u(nc)
	real      delta_old,delta_new        ! binary skewness indicator (BSI)
	real      sigma2_old,sigma2_new      ! The coefficients of epsilons (Inverse-Gamma priors)
	real      epsi_old(nc),epsi_new(nc)  ! epsilon part of random effects 
	real      z_old(nc),z_new(nc)        ! Z+ part of random effects
	real      beta_old(p),beta_new(p),cov(p,p),rc(p,p),wnb(nr,p)
	real      delta,sigma2,epsi(nc),z(nc),bet(p),value_old,value_new
	!  For identiability concern: delta could only be +1 or -1. Let Z variable
      real      w,w1,w2
	real      alpha_sigma2,beta_sigma2,
     &	      mu_Z,sigma_Z,mu_epsi,sigma_epsi
      real      u1(nr),u2(nr),u3(nr),u4(nc*nr),u5(nc*nr),u6(nr),u7(nr),
     &	      u_sigma2(nr),u_delta(nr),u_z(nc*nr),u_epsi(nc*nr)
      

	common /x/     x
      common /mny/   m,n,y
      common /prior/ mu_delta,sigma_delta,alpha_sigma2,beta_sigma2,
     &	           mu_Z,sigma_Z,mu_epsi,sigma_epsi

      alpha_sigma2=3
	beta_sigma2=100
      
	mu_delta=0.0
      sigma_delta=1.0

	mu_z=0.0
c      sigma_z=0.0001
      sigma_z=1.0        !   simulated standard folded normal distributions
      
	mu_epsi=0.0
c      sigma_epsi=0.01
      sigma_epsi=0.1     !   Simulated dispersion of epsilons

c	m(1)=4        ! # of binomials for cluster 1
c	m(2)=4        ! # of binomials for cluster 2
c	m(3)=4        ! # of binomials for cluster 3
c	m(4)=4        ! # of binomials for cluster 4

      do i=1,nc
	   m(i)=4
	end do

	do i=1,p
  	   do j=1,p
	      cov(i,j)=0.0
	      if(i.eq.j)cov(i,j)=0.01
	   end do
	end do
      
      open(7,file='Simu-Data-Non-Skew.txt',status='old')
      do i=1,nc 
      
	do k=1,m(i)
	   read(7,*)w1,w2,(x(i,k,j),j=1,p),n(i,k),y(i,k) 
	end do
      
	end do 
	close(7)

      iseed=13
      call chfac(p,cov,p,0.00001,irank,rc,p)

	delta_old=1.0
c     sigma2_old=0.01
      sigma2_old=1.00
	
	do i=1,nc
	   epsi_old(i)=0.0
	   z_old(i)=0.0                 ! folded N(0,1)
	end do
	
	do j=1,p
	   beta_old(j)=0.0              ! Intercept + Age
	end do

      open(2,file='Delta-Sigma2.txt',status='unknown')
      open(3,file='Z.txt',status='unknown')
      open(4,file='Epsilon.txt',status='unknown')
      open(5,file='Beta.txt',status='unknown')

      nd=0
   
1     iseed=iseed+2                      !  for updating beta vector
	call rnset(iseed)
	call rnmvn(nr,p,rc,p,wnb,nr)
	
	iseed=iseed+2
      call rnset(iseed)
	call rnun(nr,u1)
	
	iseed=iseed+2
      call rnset(iseed)
	call rnun(nr,u2)

	iseed=iseed+2
      call rnset(iseed)
	call rnun(nr,u3)

	iseed=iseed+2
      call rnset(iseed)
	call rnun(nc*nr,u4)

	iseed=iseed+2
      call rnset(iseed)
	call rnun(nc*nr,u5)

	iseed=iseed+2
      call rnset(iseed)
	call rnun(nr,u6)
	
	iseed=iseed+2
      call rnset(iseed)
	call rngam(nr,alpha_sigma2,u_sigma2)
      
	do j=1,nr
	   u_sigma2(j)=u_sigma2(j)*beta_sigma2
	   u_sigma2(j)=1.0/u_sigma2(j)
	end do

	iseed=iseed+2
      call rnset(iseed)
	call rnun(nr,u7)
	do i=1,nr
	   if(u7(i).ge.0.5)u_delta(i)= 1.0   ! indentical independent proposal from prior distribution U[(-1),(1)]
	   if(u7(i).lt.0.5)u_delta(i)=-1.0   
      end do

	iseed=iseed+2
      call rnset(iseed)
	do i=1,nc*nr
	   u_z(i)=abs(rnnof())*sigma_z       ! indentical independent proposal from prior distribution folded N(0,1)
      end do

	iseed=iseed+2
      call rnset(iseed)
	do i=1,nc*nr
	   u_epsi(i)=rnnof()*sigma_epsi       ! indentical independent proposal from prior distribution
      end do

	do i=1,nr
         nd=nd+1
         sigma2_old=1.0                     ! normal random effects has constant coefficient 1.0

c    calculate the old log-likelihood value for beta_old
	   
         call log_likelihood(beta_old,delta_old,sigma2_old,
     &	                   z_old,epsi_old,value_old)

c    propose new beta candidate

         do j=1,p
	      beta_new(j)=beta_old(j)+wnb(i,j)
         end do

c    calculate the new log-likelihood value for beta_new

	   call log_likelihood(beta_new,delta_old,sigma2_old,z_old,
     &	                   epsi_old,value_new)

         if(log(u1(i)).le.min(0.0,value_new-value_old))then
            do j=1,p
  	         beta_old(j)=beta_new(j)
            end do
	   end if

c    calculate the old log-likelihood value for delta_old

	   call log_likelihood(beta_old,delta_old,sigma2_old,z_old,
     &	                   epsi_old,value_old)

c    propose new delta candidate

         delta_new=u_delta(i)

c    calculate the new log-likelihood value for delta_new

	   call log_likelihood(beta_old,delta_new,sigma2_old,
     & 	                   z_old,epsi_old,value_new)

         if(log(u2(i)).le.min(0.0,value_new-value_old) )then
  	      delta_old=delta_new
	   end if

c     No sigma parameter multiplied by ``epsilon_i"s

         do j=1,nc   

c    calculate the old log-likelihood value for z_old

	      call log_likelihood(beta_old,delta_old,sigma2_old,
     &	                   z_old,epsi_old,value_old)

c    propose new z candidate

            do k=1,nc
	         z(k)=z_old(k)   
	      end do

            z_new(j)=u_z(nc*(i-1)+j)
	      z(j)=z_new(j)

c    calculate the new log-likelihood value for z_new

	      call log_likelihood(beta_old,delta_old,sigma2_old,
     &		                  z,epsi_old,value_new)


            value_new=value_new-0.5*(z_old(j))**2/sigma_z
     &		       -log(sqrt(sigma_z))

            value_old=value_old-0.5*(z(j))**2/sigma_z
     &		       -log(sqrt(sigma_z))

            if(log(u4(nc*(i-1)+j)).le.min(0.0,value_new-value_old) )then
  	         z_old(j)=z_new(j)
	      end if
    
	   end do

         do j=1,nc   

c    calculate the old log-likelihood value for epsi_old

	      call log_likelihood(beta_old,delta_old,sigma2_old,
     &		                  z_old,epsi_old,value_old)

c    propose new epsi candidate

            do k=1,nc
	         epsi(k)=epsi_old(k)   
	      end do

            epsi_new(j)=u_epsi(nc*(i-1)+j)
	      epsi(j)=epsi_new(j)

c    calculate the new log-likelihood value for epsilon_new

	      call log_likelihood(beta_old,delta_old,sigma2_old,
     &		                  z_old,epsi,value_new)

            w=sigma_epsi

            value_new=value_new-0.5*(epsi_old(j)/w)**2-log(sqrt(w))

            value_old=value_old-0.5*(epsi_new(j)/w)**2-log(sqrt(w))

            if(log(u5(nc*(i-1)+j)).le.min(0.0,value_new-value_old) )then
  	         epsi_old(j)=epsi_new(j)
	      end if
         end do
   	   if(mod(nd,5000).eq.0)then
            write(*,*)nd/5000
	      write(*,100)beta_old
	      write(*,100)epsi_old
	      write(*,100)z_old
	      write(*,100)delta_old,sigma2_old
	      write(2,101)delta_old,sigma2_old
	      write(3,101)z_old
	      write(4,101)epsi_old
	      write(5,101)beta_old
	   end if 

      end do

	if(nd.lt.2D08)goto 1 

      close(2)
      close(3)
      close(4)
      close(5)

100   format(15f6.1)
101   format(15f15.6)

	end

c     subroutine for calculating log-posterior function

      subroutine   log_likelihood(bet,delta,sigma2,z,epsi,value)

      integer      p,nc
	parameter    (p=2,nc=8)
	integer      m(nc),n(nc,4),y(nc,4)
	real         x(nc,4,p)
	real         u(nc),delta,sigma2,epsi(nc),z(nc),bet(p),value
      real         w
	real         mu_delta,sigma_delta,alpha_sigma2,beta_sigma2,
     &	         mu_Z,sigma_Z,mu_epsi,sigma_epsi

	common /x/   x
      common /mny/ m,n,y
      common /prior/ mu_delta,sigma_delta,alpha_sigma2,beta_sigma2,
     &	           mu_Z,sigma_Z,mu_epsi,sigma_epsi

	value=0.0

c     likelihood function part
	
	do i=1,nc
	   u(i)=delta*z(i)+sqrt(sigma2)*epsi(i)
	   do j=1,m(i)
	      w=0.0
		  do k=1,p
	         w=w+x(i,j,k)*bet(k)
		  end do 
	      w=w+u(i)
	      value=value+y(i,j)*w-n(i,j)*log(1.0+exp(w))   
	   end do
	end do

c     prior part
c     delta

      value=value-0.5

c     Z: 1 ~ nc, folded normal density

      do i=1,nc
         value=value-0.5*((z(i)-mu_Z)/sigma_z)
     &	              *((z(i)-mu_Z)/sigma_z)
      end do

c     epsilon: 1 ~ nc

      do i=1,nc
         value=value-0.5*((epsi(i)-mu_epsi)/sigma_epsi)
     &	              *((epsi(i)-mu_epsi)/sigma_epsi)
      end do

	return
	end